from authserv import app
from authserv import auth
from authserv import protocol
from authserv.ratelimit import blacklist, ratelimit
from flask import Flask, request, abort, make_response


@blacklist([0], count=5, period=600, ttl=43200)
@blacklist([4], count=5, period=600, ttl=43200)
def _auth(username, service, password, otp_token, source_ip):
    user = app.userdb.get_user(username, service)
    if not user:
        return protocol.ERR_AUTHENTICATION_FAILURE
    return auth.authenticate(user, service, password, otp_token)


@app.route('/api/v1/auth', methods=('POST',))
@ratelimit(count=10, period=60, header='HTTP_X_FORWARDED_FOR')
@ratelimit(count=10, period=60, param='username')
def do_auth():
    service = request.form.get('service')
    username = request.form.get('username')
    password = request.form.get('password')
    otp_token = request.form.get('otp')
    source_ip = request.form.get('source_ip')

    if not service or not username:
        abort(400)

    try:
        result = _auth(username, service, password, otp_token, source_ip)
    except Exception, e:
        app.logger.exception('Unexpected exception in authenticate()')
        abort(500)

    app.logger.info(
        'AUTH %s %s otp=%s %s',
        username, service, otp_token and 'y' or 'n', result)

    response = make_response(result)
    response.headers['Cache-Control'] = 'no-cache'
    response.headers['Content-Type'] = 'text/plain'
    response.headers['Expires'] = '-1'
    return response


def create_app(userdb=None, mc=None):
    app.config.from_envvar('APP_CONFIG', silent=True)

    if not userdb:
        from authserv import ldap_model
        userdb = ldap_model.UserDb(
            app.config['LDAP_SERVICE_MAP'],
            app.config.get('LDAP_URI', 'ldap://127.0.0.1:389'),
            app.config['LDAP_BIND_DN'],
            app.config['LDAP_BIND_PW'])
    app.userdb = userdb

    if not mc:
        import memcache
        mc = memcache.Client(
            app.config['MEMCACHE_ADDR'], debug=0)
    app.memcache = mc

    return app
